/*
 * gedit-scriptexecute-plugin.h - Make executable a script.
 *
 * Copyright (C) 2009-2010 - Romain MARS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gedit-executescript-plugin.h"

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

#include <glib/gi18n-lib.h>
#include <gedit/gedit-debug.h>

#include <gedit/gedit-commands.h>
#include <gedit/gedit-commands-file.c>

#define WINDOW_DATA_KEY	"GeditExecutescriptPluginWindowData"

#define GEDIT_EXECUTESCRIPT_PLUGIN_GET_PRIVATE(object)	(G_TYPE_INSTANCE_GET_PRIVATE ((object), TYPE_GEDIT_EXECUTESCRIPT_PLUGIN, GeditExecutescriptPluginPrivate))

struct _GeditExecutescriptPluginPrivate
{
	gpointer dummy;
};

GEDIT_PLUGIN_REGISTER_TYPE (GeditExecutescriptPlugin, gedit_executescript_plugin)

static void 
make_executable(GeditWindow *window,
		char *path)
{
	GtkWidget *dialog=0;
	GtkTextBuffer* text_buffer=0;
	GtkTextIter start;
	GtkTextIter end;
	gchar* buffer=0;
	GtkTextView *view=0;
	
	view = GTK_TEXT_VIEW (gedit_window_get_active_view (window));

	if (view != NULL)
	{
		text_buffer = gtk_text_view_get_buffer (view);
		gtk_text_buffer_get_start_iter(text_buffer,&start);
		gtk_text_buffer_get_end_iter(text_buffer,&end);
		buffer=gtk_text_buffer_get_text(text_buffer,&start, &end,TRUE);
	
		if (buffer != NULL) {
			int i=0;
			while ((buffer[i]==' ') || (buffer[i]=='\n') || (buffer[i]=='\t')) {
				i++;
			}
			if ((buffer[i]=='#') && (buffer[i+1]=='!')) {
				/*create modal dialog*/
				dialog = gtk_message_dialog_new (GTK_WINDOW(window),
								GTK_DIALOG_MODAL,
								GTK_MESSAGE_QUESTION,
								GTK_BUTTONS_YES_NO,
								N_("Would you like to make this script executable?"));
					
				/*waiting user response*/ 
				if (gtk_dialog_run(GTK_DIALOG(dialog))==GTK_RESPONSE_YES) {
					/*recovery current access rights*/ 
					struct stat *right;
					right= (struct stat *)malloc(sizeof(struct stat));
					stat(path,right);
					/*add execution rights*/
					chmod(path,right->st_mode & (S_ISUID | S_ISGID | S_ISVTX | S_IRWXU | S_IRWXG | S_IRWXO) | S_IXUSR | S_IXGRP | S_IXOTH);
					free(right);
				}
				gtk_widget_destroy(dialog);	
			}	
			g_free(buffer);
		}
	}
}

static void 
my_save_dialog_response_cb (GeditFileChooserDialog *dialog,
                            gint response_id,
                            GeditWindow *window)
{
	GFile *file;
	const GeditEncoding *encoding;
	GeditTab *tab;
	gpointer data;
	GSList *tabs_to_save_as;
	GeditDocumentNewlineType newline_type;

	gedit_debug (DEBUG_COMMANDS);

	tab = GEDIT_TAB (g_object_get_data (G_OBJECT (dialog),
					    GEDIT_TAB_TO_SAVE_AS));

	if (response_id != GTK_RESPONSE_OK)
	{
		gtk_widget_destroy (GTK_WIDGET (dialog));
		goto save_next_tab;
	}

	file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
	g_return_if_fail (file != NULL);

	encoding = gedit_file_chooser_dialog_get_encoding (dialog);
	newline_type = gedit_file_chooser_dialog_get_newline_type (dialog);

	gtk_widget_destroy (GTK_WIDGET (dialog));
	
	if (tab != NULL)
	{
		GeditDocument *doc;
		gchar *parse_name;
		gchar *uri;

		doc = gedit_tab_get_document (tab);
		g_return_if_fail (GEDIT_IS_DOCUMENT (doc));

		parse_name = g_file_get_parse_name (file);

		gedit_statusbar_flash_message (GEDIT_STATUSBAR (window->priv->statusbar),
					        window->priv->generic_message_cid,
					       _("Saving file '%s'\342\200\246"),
					       parse_name);

		g_free (parse_name);

		/* let's remember the dir we navigated too,
		 * even if the saving fails... */
		 _gedit_window_set_default_location (window, file);

		// FIXME: pass the GFile to tab when api is there
		uri = g_file_get_uri (file);
		_gedit_tab_save_as (tab, uri, encoding, newline_type);
		g_free (uri);
		
		
		/*Call my function which make execute a script*/
		char * path = g_file_get_path (file);	
		make_executable(window,path);
	}

	g_object_unref (file);
	
save_next_tab:

	data = g_object_get_data (G_OBJECT (window),
				  GEDIT_LIST_OF_TABS_TO_SAVE_AS);
	if (data == NULL)
		return;

	/* Save As the next tab of the list (we are Saving All files) */
	tabs_to_save_as = (GSList *)data;
	g_return_if_fail (tab == GEDIT_TAB (tabs_to_save_as->data));

	/* Remove the first item of the list */
	tabs_to_save_as = g_slist_delete_link (tabs_to_save_as,
					       tabs_to_save_as);

	g_object_set_data (G_OBJECT (window),
			   GEDIT_LIST_OF_TABS_TO_SAVE_AS,
			   tabs_to_save_as);

	if (tabs_to_save_as != NULL)
	{
		tab = GEDIT_TAB (tabs_to_save_as->data);

		if (GPOINTER_TO_BOOLEAN (g_object_get_data (G_OBJECT (tab),
							    GEDIT_IS_CLOSING_TAB)) == TRUE)
		{
			g_object_set_data (G_OBJECT (tab),
					   GEDIT_IS_CLOSING_TAB,
					   NULL);

			/* Trace tab state changes */
			g_signal_connect (tab,
					  "notify::state",
					  G_CALLBACK (tab_state_changed_while_saving),
					  window);
		}

		gedit_window_set_active_tab (window, tab);
		file_save_as (tab, window);
	}
}

static void
my_file_save_as (GeditTab    *tab,
	      	 GeditWindow *window)
{
	GtkWidget *save_dialog;
	GtkWindowGroup *wg;
	GeditDocument *doc;
	GFile *file;
	gboolean uri_set = FALSE;
	const GeditEncoding *encoding;
	GeditDocumentNewlineType newline_type;

	g_return_if_fail (GEDIT_IS_TAB (tab));
	g_return_if_fail (GEDIT_IS_WINDOW (window));

	gedit_debug (DEBUG_COMMANDS);

	save_dialog = gedit_file_chooser_dialog_new (_("Save As\342\200\246"),
						     GTK_WINDOW (window),
						     GTK_FILE_CHOOSER_ACTION_SAVE,
						     NULL,
						     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
						     GTK_STOCK_SAVE, GTK_RESPONSE_OK,
						     NULL);

	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (save_dialog),
							TRUE);
	g_signal_connect (save_dialog,
			  "confirm-overwrite",
			  G_CALLBACK (confirm_overwrite_callback),
			  NULL);

	wg = gedit_window_get_group (window);

	gtk_window_group_add_window (wg,
				     GTK_WINDOW (save_dialog));

	/* Save As dialog is modal to its main window */
	gtk_window_set_modal (GTK_WINDOW (save_dialog), TRUE);

	/* Set the suggested file name */
	doc = gedit_tab_get_document (tab);
	file = gedit_document_get_location (doc);

	if (file != NULL)
	{
		uri_set = gtk_file_chooser_set_file (GTK_FILE_CHOOSER (save_dialog),
						     file,
						     NULL);

		g_object_unref (file);
	}


	if (!uri_set)
	{
		GFile *default_path;
		gchar *docname;

		default_path = _gedit_window_get_default_location (window);
		docname = gedit_document_get_short_name_for_display (doc);

		if (default_path != NULL)
		{
			gchar *uri;

			uri = g_file_get_uri (default_path);
			gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (save_dialog),
								 uri);

			g_free (uri);
			g_object_unref (default_path);
		}

		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (save_dialog),
						   docname);

		g_free (docname);
	}

	/* Set suggested encoding */
	encoding = gedit_document_get_encoding (doc);
	g_return_if_fail (encoding != NULL);

	newline_type = gedit_document_get_newline_type (doc);

	gedit_file_chooser_dialog_set_encoding (GEDIT_FILE_CHOOSER_DIALOG (save_dialog),
						encoding);

	gedit_file_chooser_dialog_set_newline_type (GEDIT_FILE_CHOOSER_DIALOG (save_dialog),
	                                            newline_type);

	g_object_set_data (G_OBJECT (save_dialog),
			   GEDIT_TAB_TO_SAVE_AS,
			   tab);

	g_signal_connect (save_dialog,
			  "response",
			  G_CALLBACK (my_save_dialog_response_cb),
			  window);

	gtk_widget_show (save_dialog);
}

static void
my_file_save (GeditTab    *tab,
	      GeditWindow *window)
{
	GeditDocument *doc;
	gchar *uri_for_display;

	gedit_debug (DEBUG_COMMANDS);

	g_return_if_fail (GEDIT_IS_TAB (tab));
	g_return_if_fail (GEDIT_IS_WINDOW (window));

	doc = gedit_tab_get_document (tab);
	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));

	if (gedit_document_is_untitled (doc) || 
	    gedit_document_get_readonly (doc))
	{
		gedit_debug_message (DEBUG_COMMANDS, "Untitled or Readonly");

		my_file_save_as (tab, window);		
		return;
	}

	uri_for_display = gedit_document_get_uri_for_display (doc);
	gedit_statusbar_flash_message (GEDIT_STATUSBAR (window->priv->statusbar),
				        window->priv->generic_message_cid,
				       _("Saving file '%s'\342\200\246"),
				       uri_for_display);

	g_free (uri_for_display);

	_gedit_tab_save (tab);
}

static void
_my_gedit_cmd_file_save_as (GtkAction   *action,
			    GeditWindow *window)
{
	GeditTab *tab;

	gedit_debug (DEBUG_COMMANDS);

	tab = gedit_window_get_active_tab (window);
	if (tab == NULL)
		return;

	my_file_save_as (tab, window);
}

static void 
_my_gedit_cmd_file_save(GtkAction *action,
			GeditWindow *window)
{
	GeditTab *tab;

	gedit_debug (DEBUG_COMMANDS);

	tab = gedit_window_get_active_tab (window);
	if (tab == NULL)
		return;

	my_file_save (tab, window);
}

static void
gedit_executescript_plugin_init (GeditExecutescriptPlugin *plugin)
{
	plugin->priv = GEDIT_EXECUTESCRIPT_PLUGIN_GET_PRIVATE (plugin);

	gedit_debug_message (DEBUG_PLUGINS, "GeditExecutescriptPlugin initializing");
}

static void
gedit_executescript_plugin_finalize (GObject *object)
{
	gedit_debug_message (DEBUG_PLUGINS, "GeditExecutescriptPlugin finalizing");

	G_OBJECT_CLASS (gedit_executescript_plugin_parent_class)->finalize (object);
}

static void
impl_activate (GeditPlugin *plugin,
	       GeditWindow *window)
{
	GtkUIManager *manager;	

	gedit_debug (DEBUG_PLUGINS);

	manager = gedit_window_get_ui_manager (window);
	
	/*disconnect the CALLBACK function "_gedit_cmd_file_save", to connect "_my_gedit_cmd_file_save"*/
	GtkAction * action =  gtk_ui_manager_get_action(manager,"/MenuBar/FileMenu/FileSaveMenu");
	if (action != NULL) {
		int handlerId = g_signal_handler_find(G_OBJECT (action), G_SIGNAL_MATCH_FUNC, 0, 0, 0, G_CALLBACK (_gedit_cmd_file_save), 0);  
		if (handlerId != 0) {
			g_signal_handler_disconnect(G_OBJECT (action), handlerId);
			g_signal_connect(G_OBJECT (action), "activate", G_CALLBACK(_my_gedit_cmd_file_save), window);
		}
	}
	/*disconnect the CALLBACK function "_gedit_cmd_file_save_as", to connect "_my_gedit_cmd_file_save_as"*/
	action =  gtk_ui_manager_get_action(manager,"/MenuBar/FileMenu/FileSaveAsMenu");
	if (action != NULL) {
		int handlerId = g_signal_handler_find(G_OBJECT (action), G_SIGNAL_MATCH_FUNC, 0, 0, 0, G_CALLBACK (_gedit_cmd_file_save_as), 0);  
		if (handlerId != 0) {
			g_signal_handler_disconnect(G_OBJECT (action), handlerId);
			g_signal_connect(G_OBJECT (action), "activate", G_CALLBACK(_my_gedit_cmd_file_save_as), window);
		}
	}	
}

static void
impl_deactivate (GeditPlugin *plugin,
		 GeditWindow *window)
{
	GtkUIManager *manager;

	gedit_debug (DEBUG_PLUGINS);
	
	manager = gedit_window_get_ui_manager (window);
	
	/*disconnect my CALLBACK function "_my_gedit_cmd_file_save", to reconnect "_gedit_cmd_file_save"*/
	GtkAction * action =  gtk_ui_manager_get_action(manager,"/MenuBar/FileMenu/FileSaveMenu");
	if (action != NULL) {
		int handlerId = g_signal_handler_find(G_OBJECT (action), G_SIGNAL_MATCH_FUNC, 0, 0, 0, G_CALLBACK (_my_gedit_cmd_file_save), 0);  
		if (handlerId != 0) {
			g_signal_handler_disconnect(G_OBJECT (action), handlerId);
			g_signal_connect(G_OBJECT (action), "activate", G_CALLBACK(_gedit_cmd_file_save), window);
		}
	}
	/*disconnect my CALLBACK function "_my_gedit_cmd_file_save_as", to reconnect "_gedit_cmd_file_save_as"*/
	action =  gtk_ui_manager_get_action(manager,"/MenuBar/FileMenu/FileSaveAsMenu");
	if (action != NULL) {
		int handlerId = g_signal_handler_find(G_OBJECT (action), G_SIGNAL_MATCH_FUNC, 0, 0, 0, G_CALLBACK (_my_gedit_cmd_file_save_as), 0);  
		if (handlerId != 0) {
			g_signal_handler_disconnect(G_OBJECT (action), handlerId);
			g_signal_connect(G_OBJECT (action), "activate", G_CALLBACK(_gedit_cmd_file_save_as), window);
		}
	}
}

static void
impl_update_ui (GeditPlugin *plugin,
		GeditWindow *window)
{
	gedit_debug (DEBUG_PLUGINS);
}

static void
gedit_executescript_plugin_class_init (GeditExecutescriptPluginClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	GeditPluginClass *plugin_class = GEDIT_PLUGIN_CLASS (klass);

	object_class->finalize = gedit_executescript_plugin_finalize;

	plugin_class->activate = impl_activate;
	plugin_class->deactivate = impl_deactivate;
	plugin_class->update_ui = impl_update_ui;

	g_type_class_add_private (object_class, sizeof (GeditExecutescriptPluginPrivate));
}
